package common

// MessageHandler 消息处理函数
// 参数:
// - queue: 消息队列
// - topic: 主题
// - data: 消息数据
// 返回值: 无
type MessageHandler func(queue MessageQueue, topic string, data string)

// MessageQueue 消息队列接口
type MessageQueue interface {
	Subscribe(group string, topic string, handler MessageHandler) error
	UnSubscribe(group string, topic string) error
	Publish(topic string, data string) error
}

type Option func(options *Options)

type Options struct {
	MaxLen      int64
	ConsumerNum int
}

func WithMaxLen(maxLen int64) Option {
	return func(options *Options) {
		options.MaxLen = maxLen
	}
}

func WithConsumerNum(consumerNum int) Option {
	return func(options *Options) {
		options.ConsumerNum = consumerNum
	}
}
